/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */

/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
 */

#ifndef MUTT_LIB_LIB_LIST_H
#define MUTT_LIB_LIB_LIST_H

#define DO_SLIST(type, prefix, dtor)                                         \
    static inline type *prefix##_list_pop(type **list) {                     \
        if (*list) {                                                         \
            type *res = *list;                                               \
            *list = res->next;                                               \
            res->next = NULL;                                                \
            return res;                                                      \
        }                                                                    \
        return NULL;                                                         \
    }                                                                        \
    static inline void prefix##_list_push(type **list, type *item) {         \
        item->next = *list;                                                  \
        *list = item;                                                        \
    }                                                                        \
                                                                             \
    static inline type **prefix##_list_last(type **list) {                   \
        while (*list) {                                                      \
            list = &(*list)->next;                                           \
        }                                                                    \
        return list;                                                         \
    }                                                                        \
                                                                             \
    static inline type **prefix##_list_append(type **list, type *item) {     \
        list = prefix##_list_last(list);                                     \
        *list = item;                                                        \
        return list;                                                         \
    }                                                                        \
                                                                             \
    static inline type **prefix##_list_init(type **list) {                   \
        *list = NULL;                                                        \
        return list;                                                         \
    }                                                                        \
    static inline void prefix##_list_wipe(type **list) {                     \
        while (*list) {                                                      \
            type *item = prefix##_list_pop(list);                            \
            dtor(&item);                                                     \
        }                                                                    \
    }                                                                        \


typedef struct string_list_t {
    struct string_list_t *next;
    char *data;
} string_list_t;

DO_INIT(string_list_t, string_item);
static inline void string_item_wipe(string_list_t *it) {
    p_delete(&it->data);
}
DO_NEW(string_list_t, string_item);
DO_DELETE(string_list_t, string_item);
DO_SLIST(string_list_t, string, string_item_delete);

string_list_t *string_list_dup(const string_list_t *);

/* FIXME: b0rken API's, replace that at any cost */
/* add an element to a list */
string_list_t *mutt_add_list_n(string_list_t*, const void*, size_t len);
static inline string_list_t *mutt_add_list(string_list_t *head, const char *data) {
    size_t len = m_strlen(data);
    return mutt_add_list_n(head, data, len ? len + 1 : 0);
}

#endif /* MUTT_LIB_LIB_LIST_H */
